@@ -380,9 +380,9 @@ class Link:
         else:
             rp = ""
         if self.comes_from:
-            return f"{redact_auth_from_url(self._url)} (from {self.comes_from}){rp}"
+            return f"{self.redacted_url} (from {self.comes_from}){rp}"
         else:
-            return redact_auth_from_url(str(self._url))
+            return self.redacted_url
 
     def __repr__(self) -> str:
         return f"<Link {self}>"
@@ -404,6 +404,10 @@ class Link:
     def url(self) -> str:
         return self._url
 
+    @property
+    def redacted_url(self) -> str:
+        return redact_auth_from_url(self.url)
+
     @property
     def filename(self) -> str:
         path = self.path.rstrip("/")
@@ -468,7 +472,7 @@ class Link:
             deprecated(
                 reason=f"{self} contains an egg fragment with a non-PEP 508 name.",
                 replacement="to use the req @ url syntax, and remove the egg fragment",
-                gone_in="25.1",
+                gone_in="25.2",
                 issue=13157,
             )
 
